#define G_LOG_DOMAIN "gpod-feed-parser"

#include "config.h"

#include "gpod-feed.h"
#include "gpod-feed-item.h"
#include "gpod-feed-parser.h"

#include <libsoup/soup.h>

#include <libxml/tree.h>
#include <libxml/parser.h>

#include <gio/gio.h>
#include <glib.h>
#include <glib-object.h>

/**
 * SECTION:gpod-feed-parser
 * @title: GpodFeedParser
 * @short_description: A #GtkBox widget for list feeds
 * @include: gpod-feed-parser.h
 * @see_also: #GpodFeed, #GpodFeedWidget
 *
 * The #GpodFeedParser is an instance #GtkBox, and acts as the primary
 * window/widget for listing feeds in the #GpodWindow<!-- -->'s #GtkStack.
 **/

struct _GpodFeedParser
{
    GObject parent;
};

G_DEFINE_TYPE (GpodFeedParser, gpod_feed_parser, G_TYPE_OBJECT)

GInputStream *
gpod_feed_parser_real_download_data_from_uri (GpodFeedParser *self,
                                              gchar           *uri)
{
    g_assert (GPOD_IS_FEED_PARSER (self));
    g_return_val_if_fail (uri != NULL, NULL);

    g_debug ("Downloading feed from uri \"%s\"", uri);

    SoupSession *session;
    GError *error = NULL;
    SoupRequest *request;
    GInputStream *feed;

    session = soup_session_new ();

    request = soup_session_request (session, uri, &error);

    if (error)
    {
        g_warning ("Unable to download feed from uri \"%s\" because: %s",
                   uri, error->message);

        g_error_free (error);

        return NULL;
    }

    feed = soup_request_send (request, NULL, &error);

    if (error)
    {
        g_warning ("Unable to retrieve stream feed from uri \"%s\" because: %s",
                   uri, error->message);

        g_error_free (error);

        return NULL;
    }

    return feed;
}

void
gpod_feed_parser_real_save_data_from_uri (GpodFeedParser *self,
                                          gchar          *uri,
                                          gchar          *filename)
{
    g_assert (GPOD_IS_FEED_PARSER (self));
    g_return_if_fail (uri != NULL);
    g_return_if_fail (filename != NULL);

    GInputStream *feed_data;
    GError *error = NULL;
    gchar *buffer[4000];

    feed_data = gpod_feed_parser_real_download_data_from_uri (self, uri);

    g_input_stream_read (feed_data,
                         buffer,
                         sizeof (buffer),
                         NULL,
                         &error);

    g_print ("%s", buffer);
}

GpodFeed *
gpod_feed_parser_real_parse_xml_from_uri (GpodFeedParser *self,
                                          gchar          *uri)
{
    g_assert (GPOD_IS_FEED_PARSER (self));
    g_return_val_if_fail (uri != NULL, NULL);

    GInputStream *feed_data;
    GpodFeed *feed;

    feed_data = gpod_feed_parser_real_download_data_from_uri (self, uri);

    if (feed_data == NULL)
    {
        g_warning ("No feed data to parse from uri: \"%s\"", uri);
        return NULL;
    }

    feed = gpod_feed_new ();


    //g_input_stream_read (stream, buffer, sizeof buffer, NULL, NULL);


    return feed;
}

/*
 * gpod_feed_parser_finalize:
 * @object: An instance of #GpodFeedParser
 *
 * Does any last minute clean-ups when the class ends.
 */
static void
gpod_feed_parser_finalize (GObject *object)
{
    G_OBJECT_CLASS (gpod_feed_parser_parent_class)->finalize(object);
}

/*
 * gpod_feed_parser_init:
 * @self: An instance of #GpodFeedParser
 *
 * Initializes various aspects of the #GpodFeedParser. Anything the #GpodFeedParser
 * would need before becoming presented should be done here.
 */
static void
gpod_feed_parser_init (GpodFeedParser *self)
{

}

/*
 * gpod_feed_parser_class_init:
 * @klass: An instance of #GpodFeedParserClass
 *
 * Initializes the various virtural functions in #GpodFeedParserClass. In
 * addtition it connects the various #GtkWidgets from the gresource file using
 * gtk_widget_class_set_template_from_resource().
 */
static void
gpod_feed_parser_class_init (GpodFeedParserClass *klass)
{
    GObjectClass *object_class = G_OBJECT_CLASS (klass);

    object_class->finalize = gpod_feed_parser_finalize;
}

/**
 * gpod_feed_parser_new:
 *
 * Creates a new #GpodFeedParser, a custom widget to show all feeds user is
 * subscribed to.
 *
 * ## Usage
 * |[
 * GpodFeedParser *feed_parser;
 *
 * feed_parser = gpod_feed_parser_new ();
 * ]|
 *
 * Returns: #GpodFeedParser
 */
GpodFeedParser *
gpod_feed_parser_new ()
{
    return g_object_new (GPOD_TYPE_FEED_PARSER,
                         NULL);
}

GInputStream *
gpod_feed_parser_download_data_from_uri (GpodFeedParser *self,
                                         gchar           *uri)
{
    return gpod_feed_parser_real_download_data_from_uri (self, uri);
}


void
gpod_feed_parser_save_data_from_uri (GpodFeedParser *self,
                                     gchar          *uri,
                                     gchar          *filename)
{
    gpod_feed_parser_real_save_data_from_uri (self, uri, filename);
}

GpodFeed *
gpod_feed_parser_parse_xml_from_uri (GpodFeedParser *self,
                                     gchar          *uri)
{
    return gpod_feed_parser_real_parse_xml_from_uri (self, uri);
}
